# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004-2007 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

###################
#
# TODO Liste fr das gesamte Projekt
#
#   * Abschluss OGR-Export -> Tests schreiben
#   * Wahlmglichkeit des Interfaces im Hauptmen
#   * Export als Methoden an die einzelnen Feature und Objektteile
#      implementieren
#   * berarbeitung der Systems fr Fehlerberichte
#       -> benutze 'raise' statt 'logging'
#   * punkttrans soll nicht mehr in exportXX aufzurufen sein
#
##################

import sys
import os
sys.path +=[os.path.join(os.pardir, "Lib")]

import logging

from parse import parsen, TableConf

edbsilon_version = "0.9.1"

"""
edbsilon.py -- Konvertierungsprogramm fr EDBS-Daten Befehl:

  python edbsilon [ -o <Ziel der SQL_Kom>] [-l <Ziel des Logbuchs> ]
            [-k <Konfigurationsdatei>] Dateien....   

Die Dateien sind die EDBS-Quelldateien, die abgearbeitet werden sollen.

Vorgehensweise beim Konvertiern: 
   1. Parsing: Suche nach EDBS-Datenstzen, die Zusammen gehren.
      Dies wird durch das A F E in den Zugehrigkeitsbyte angegeben. 
      Jeder EDBS-Satz wird in ein EDBS_satz -Objekt kopiert.
      Stze, die zusammen gehren, werden in ein EDBS_satz_liste -Objekt
      geschoben. Dabei bleibt die Reihenfolge erhalten.
      In EDBS_satz_liste werden diese EDBS_satz-Objekte in der Liste
      list gespeichter.
      Die EDBS_satz -Objekte und die
      EDBS_satz_liste -Objekte werden in der Liste plist
      eingehngt.

      z.B [E,  EL,E,...,E] 0 plist
           ^    ^
           |    |    E means an objekt of EDBS_satz
           |    two or more
           |     EDBS-data-lines, which belong together, are stored in 
           |     edbs_satz_liste
           a standalone EDBS-data-line 

   2. Parsing: Geht die Liste aus edbs_satz und edbs_satz_liste -Objekte durch:
               Falls OP und IN folgendes sind, mache:
               BSPE ULOBNN.. : Grundrissdatenverarbeiten
               FEIN ULOBNN.. : Grundrissdatenverarbeiten 
               BINF ULOTEX.. : Text ins Logbuch schreiben
               OTEX ULOTEX.. : "
               BKRT IBENKRT. : Gebietskennzeichen ins Logbuchschreiben
               AKND ULQA0000 : Auftragskennsatz ins Logbuch schreiben
               AEND ........ : Ende ins Logbuch schreiben   
   3. Wegschreiben: Das Die ONS wird durchgegangen und bei den enthalten 
        Feature -Objekte wird die Objektnummer kontrolliert mit compareObjnr
        und dann wird durch wandel() die Informationen in SQL verpackt."""


def hilfeausgeben():
    """mit print wird die Hilfe ausgegeben"""

    print "edbsilon %s -- Konvertiert EDBS-Daten" % edbsilon_version
    print "Benutzung: python edbsilon.py [Optionen] Quelldatein..."
    print "Optionen: -o <Ziel>         -- SQL-Kommandos [Default: edbsilon.sql]"
    print "          -k <Konffile>     -- Konfigurationsdatei [Default: alk.konf]"
    print "          -l <Ziel_Logbuch> -- Logbuch [Default: edbsilon.log]" 
    print "          -f <Format>       -- Format [Default: ORACLE]" 
    print "          -a [alk|atkis]    -- Setzt EDBS-Format auf ALK "
    print "                               bzw.  ATKIS [Default: alk]"
    print "          -h                -- fr diese Hilfe"


# Diese Befehle laufen beim Hauptaufruf.    
if __name__ == "__main__":

    verbraucht = 0 

    format = 'Oracle'
    ziel = 'edbsilon.sql'
    logbuch = 'edbsilon.log'
    Konfdatei = 'alk.konf'
    edbsformat = 'alk'
    Quell_Datei_Liste = []
    # Die Argumente des aufrufes verstehen 
    for n in range(len (sys.argv)):
        arg = sys.argv[n]
        if n == 0:
            continue
        elif verbraucht == 1:
            verbraucht = 0
            continue
        elif arg[0:2] == "-h":
            hilfeausgeben()
            sys.exit()     
        elif arg[0:2] == "-o":
            verbraucht = 1
            if n+1 < len (sys.argv): 
                ziel = sys.argv[n+1]
            else:
                hilfeausgeben()
                sys.exit(1)    
        elif arg[0:2] == "-l":
            verbraucht = 1
            if n+1 < len (sys.argv):
                logbuch = sys.argv[n+1]
            else:
                hilfeausgeben()
                sys.exit(1)  
        elif arg[0:2] == "-f":
            verbraucht = 1
            if n+1 < len (sys.argv):
                format = sys.argv[n+1]
            else:
                hilfeausgeben()
                sys.exit(1)  
        elif arg[0:2] == "-a":
            verbraucht = 1
            if n+1 < len (sys.argv):
                edbsformat = sys.argv[n+1].lower()
            else:
                hilfeausgeben()
                sys.exit(1)  
        elif arg[0:2] == "-k":
            verbraucht = 1
            if n+1 < len (sys.argv):
                Konfdatei = sys.argv[n+1]
            else:
                hilfeausgeben()
                sys.exit(1)
        else:
            Quell_Datei_Liste += [ arg ]
    
    # Ist keine Quelle angegeben: Hilfe ausgeben
    if Quell_Datei_Liste == []:
        hilfeausgeben() 
        sys.exit()
    
    log = logging.getLogger('')
    try:
        hdlr = logging.FileHandler(logbuch)
    except:
        print "Logbuch konnte nicht geffnet werden."
        hilfeausgeben()
        sys.exit()

    log_format = logging.Formatter('%(message)s')
    hdlr.setFormatter(log_format)
    log.addHandler(hdlr) 
    log.setLevel(logging.INFO)

    # Weitere Datenformate mssen ebenfalls eine FeatureCollection mit
    # einem leeren {} als ONS sowie der Methode export implementieren.
    if format == 'Oracle':
        from exportDB import FeatureCollection
        fc = FeatureCollection()
    else:
        from exportOGR import FeatureCollection
        fc = FeatureCollection(driver = format)

    # Einlesen der Konfigurationsdatei
    print "Lese Konfigurationsdatei"
    fc.tableconf = TableConf(Konfdatei)
    fc.datasource = ziel

    fc.createSchema()
    fc.dropSchema()
    # Parsen und Exportieren der EDBS-Daten aus allen Quelldateien
    for quelle in Quell_Datei_Liste:
        print "Lese", quelle
        parsen(fc.ONS, quelle, edbsformat)
        fc.export()
        fc.ONS = {}

        ## Ausgabe, hier als SQL-Kommandos
        #if edbsclasses.format in ('Oracle', 'PG'):
            #exportDB.exportiere_SQL(ONS , sqlfile, tableconf)
        #if edbsclasses.format == 'OGR':
            #ogrCollection = exportOGR.OGRCollection(tableconf.tables.keys())
            #ogrCollection.readONS(ONS)
            #ogrCollection.export()
