# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""test_exportDB.py    von Ludwig Reiter und Silke Reimer
   begonnen: 19.08.2004
   Testet bisher: * first_parsing Funktion von der Klassen entry
                    und entrylist von konvertor.py
                  * get_II_data Funktion von der Klasse entry
                  * II_data Klasse
                  * second_parsing Funktion
                  * die Feature Klasse                 """

# Zuerst wird versucht die richtigen Pfade zu setzen
import sys
import os
from string import join, split
sys.path +=[os.path.join(os.pardir, "Konverter")]
sys.path +=[os.path.join(os.pardir, "Lib")]

import unittest
import logging

from edbsclasses import Feature, edbs_line, edbs_satz, objektTeil,\
    punkttrans, Gamma
from parse import TableConf, second_parsing
from exportDB import exportFeature, exportLine, exportPolygon,\
    exportObjektteil, FeatureCollection

log = logging.getLogger('')
hdlr = logging.FileHandler('test.log')
log_format = logging.Formatter('%(message)s')
hdlr.setFormatter(log_format)
log.addHandler(hdlr)
log.setLevel(logging.INFO)


class testCreateSQL(unittest.TestCase):

    def setUp(self):

        # Tabellenkonfiguration einlesen
        self.fc = FeatureCollection()
        self.fc.tableconf = TableConf("land.konf")


    def testCreateObjekt(self):
        """Testet, ob ein CREATE-SQL fr ein Objekt richtig erzeugt wird"""

        testsql = ["CREATE TABLE f001_f\n"]
        testsql.append("(objnr VARCHAR(7),\n")
        testsql.append("ogr_fid INTEGER,\n")
        testsql.append("objart INTEGER,\n")
        testsql.append("aktualitaet VARCHAR(2),\n")
        testsql.append("edatum DATE,\n")
        testsql.append("geom MDSYS.SDO_GEOMETRY,\n")
        testsql.append("fkz VARCHAR(17));\n")


        sqlfile = open("test.sql", 'w')
        self.fc.createSQL(sqlfile)
        sqlfile.close()
        sqllines = open("test.sql",'r').readlines()
        self.assert_(testsql[0] in sqllines, "%s not in %s" %(testsql[0], sqllines))
        sqlindex = sqllines.index(testsql[0])
        self.assertEqual(testsql, sqllines[sqlindex:sqlindex+8])


class testExportBI(unittest.TestCase):

    def testget_BesondereInfo(self):
        """Legt Feature-Objekte an und ruft die Methode
        get_BesondereInfo auf. Vergleicht das SQL_Kom der Ausgabe mit dem
        was sein sollte.
           """

        # Tabellenkonfiguration einlesen
        tables = TableConf("land.konf").tables

        ### Feature 1
        # anlegen
        col1 = Feature("000000A")
        tmp = Gamma()
        tmp.objfkt = "0010233NNP000000A123456"
        tmp.betalist = [(["13NN123456AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG  "], [])] 
        col1.setObjdaten(tmp ,(0.0,0.0))

        # exportieren
        attrSpec = \
            tables[('001', '13', '3456')]['namen']['attributes'].split(';')

        insertsql1 = exportObjektteil(col1, col1.objektteile[0], attrSpec, 'namen')

        # Vergleichsstring
        testsql1 = """BEGIN
INSERT INTO namen (objnr, kartentyp, \
darstellungsart, test) VALUES('000000A', 'NN', 3456, \
'AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG');
COMMIT;
END;
/
"""
        self.assertEqual(insertsql1, testsql1)

        ### Feature 2
        col2 = Feature("000000B")
        tmp = Gamma()
        tmp.objfkt = "0010233NNP000000B123456"
        tmp.betalist = [(["13NN123456AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG51"],\
            ["00000000000500000000"])]

        col2.setObjdaten(tmp, (0,0) )

        insertsql2 = exportObjektteil(col2, col2.objektteile[0], attrSpec, 'namen')
        testsql2 = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2001, null, MDSYS.SDO_POINT_TYPE(0.5,0.0,null)\
, null, null);
BEGIN
INSERT INTO namen_p (objnr, kartentyp, darstellungsart, test, geom) \
VALUES('000000B', 'NN', 3456, 'AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG', geom);
COMMIT;
END;
/
"""
        self.assertEqual(insertsql2, testsql2)

        ### Feature 3
        col3 = Feature("000000C")
        tmp = Gamma()
        tmp.objfkt = "0010233NNP000000C123456"
        tmp.betalist = [(["31NN123456AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG11"],\
            ["00000000000500000000","00000000000000000000"])]
        col3.setObjdaten(tmp, (0.0,1.0))
        attrSpec = \
            tables[('001', '31', '3456')]['pfeil']['attributes'].split(';')

        testsql3 = "DECLARE\ngeom mdsys.sdo_geometry :=\n\
MDSYS.SDO_GEOMETRY(2002, null, null, \
MDSYS.SDO_ELEM_INFO_ARRAY(1, 2, 1),\n\
MDSYS.SDO_ORDINATE_ARRAY(0.5,0.0,0.0,0.0));\n\
BEGIN\n\
INSERT INTO pfeil_l (objnr, kartentyp, darstellungsart, test, geom) \
VALUES('000000C', 'NN', 3456, 'AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGG', geom);\n\
COMMIT;\nEND;\n/\n"
        
        insertsql3 = exportObjektteil(col3, col3.objektteile[0], attrSpec, 'pfeil')
        self.assertEqual(insertsql3, testsql3)


class testExportGeom(unittest.TestCase):

    def setUp(self):
        """Aufbau: Legt einige line und Feature -Objekte an"""
        # (0 3),(1 2) tempLinie
        line9 = edbs_line((0.0, 3.0),(1.0, 2.0),
                       "",[],"","0242")
        # (1 2),(1 1)
        line10 = edbs_line((1.0, 2.0),(1.0, 1.0),
                       "",[],"","")
        # (1 1), (2 1)
        line11 = edbs_line((1.0, 1.0),(2.0, 1.0) ,
                       "",[],"","")
        # (2 1),(2 2)
        line12 = edbs_line((2.0, 1.0) ,(2.0, 2.0),
                       "",[],"","")
        # (2 2),(1 2)
        line13 = edbs_line((2.0, 2.0),(1.0, 2.0),
                       "",[],"","")
        # (1 2),(0 3) tempLinie
        line14 = edbs_line((1.0, 2.0),(0.0, 3.0),
                       "",[],"","0242")
        # (0 3), (3 3), (3 0), (0 0)
        line15 = edbs_line((0.0, 3.0), (0.0, 0.0),
                       "",[(3.0, 3.0),(3.0, 0.0)],"","")
        # (0 3), (3 3), (0 3)
        line16 = edbs_line((0.0, 3.0),(2.0, 0.0),
                       "",[(3.0, 3.0),(3.0, 0.0)],"","")

        self.col1 = Feature('TESTnr1')
        self.col2 = Feature('TESTnr2')
        self.col3 = Feature('TESTnr3')

        # col1: Linie mit zwei Linienzgen
        self.col1.appendline(line11)
        self.col1.appendline(line9)
        self.col1.appendline(line12)
        self.col1.appendline(line13)
        self.col1.appendline(line14)
        self.col1.appendline(line15)

        # col2: Linie mit zwei Linienzgen, deren Zwischenlinien (Art = 0242)
        # beide am Anfang stehen
        self.col2.appendline(line9)
        self.col2.appendline(line14)
        self.col2.appendline(line11)
        self.col2.appendline(line12)
        self.col2.appendline(line13)
        self.col2.appendline(line15)
        
    
    def testExportMultiLine(self):
        """ Testet export von Multiline-Objekten """
        
        orageom = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2006, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1,9,2,1),\nMDSYS.SDO_ORDINATE_ARRAY(1.0,1.0,2.0,1.0,2.0,2.0,1.0,2.0,0.0,3.0,3.0,3.0,3.0,0.0,0.0,0.0));
"""

        SQL = exportLine(self.col1)
        self.assertEqual(orageom, SQL);

    def testExportMultiLineStartTemp(self):
        """ Testet Export von Multiline mit temp-Linien am Anfang """
        
        orageom = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2006, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1,9,2,1),\nMDSYS.SDO_ORDINATE_ARRAY(1.0,1.0,2.0,1.0,2.0,2.0,1.0,2.0,0.0,3.0,3.0,3.0,3.0,0.0,0.0,0.0));
"""

        SQL = exportLine(self.col2)
        self.assertEqual(orageom, SQL);

    def testExportLargePolygonWithHole(self):
        """ Testet, ob ein groes Polygon mit Loch die Zeilenlnge von 2499 nicht berschreitet"""

        coord_list = []

        x = 2000000.001
        y = 4700000.001
        coord_list.append(x)
        coord_list.append("%s\n" %y)
        for i in range(1,8):
            y += i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x += i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y -= i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x -= i
            coord_list.append(x)
            coord_list.append(y)

        x = 2000000.031
        y = 4700000.031

        coord_list.append(x)
        coord_list.append("%s\n" %y)
        for i in range(1,8):
            x += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x -= 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y -= 1
            coord_list.append(x)
            coord_list.append(y)

        x = 2000003.001
        y = 4700003.001
        coord_list.append(x)
        coord_list.append("%s\n" %y)
        for i in range(1,8):
            x += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x -= 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y -= 1
            coord_list.append(x)
            coord_list.append(y)


        x = 3000000.001
        y = 5700000.001
        coord_list.append(x)
        coord_list.append("%s\n" %y)
        for i in range(1,8):
            y += i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x += i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y -= i
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x -= i
            coord_list.append(x)
            coord_list.append(y)

        x = 3000003.001
        y = 5700003.001
        coord_list.append(x)
        coord_list.append("%s\n" %y)
        for i in range(1,8):
            x += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y += 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            x -= 1
            coord_list.append(x)
            coord_list.append(y)
        for i in range(1,8):
            y -= 1
            coord_list.append(x)
            coord_list.append(y)
        

        orageom = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2007, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1,59,2003,1,117,2003,1,175,1003,1,233,2003,1),\nMDSYS.SDO_ORDINATE_ARRAY(%s));
""" % join(map(str,coord_list),',')


        z = long('30570000000001000001')
        for i in range(1,8):
            z1 = z + i * 1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z + i * long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z - i * 1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z - i * long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1

        z = long('30570000003001003001')
        for i in range(1,8):
            z1 = z +  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z +  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1

        z = long('30570000003001003001')
        z1 = long('30570000000001001001')
        ztrans = punkttrans(str(z), 'alk')
        z1trans = punkttrans(str(z1), 'alk')
        self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", "0242"))
        self.col3.appendline(edbs_line(z1trans, ztrans, "", [], "", "0242"))

        z = long('20470000000001000001')
        for i in range(1,8):
            z1 = z + i * 1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z + i * long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z - i * 1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z - i * long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1

        z = long('20470000003001003001')
        for i in range(1,8):
            z1 = z +  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z +  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1

        z = long('20470000003001003001')
        z1 = long('20470000000001001001')
        ztrans = punkttrans(str(z), 'alk')
        z1trans = punkttrans(str(z1), 'alk')
        self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", "0242"))
        self.col3.appendline(edbs_line(z1trans, ztrans, "", [], "", "0242"))

        z = long('20470000000031000031')
        for i in range(1,8):
            z1 = z +  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z +  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  long(1000000000)
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1
        for i in range(1,8):
            z1 = z -  1000
            ztrans = punkttrans(str(z), 'alk')
            z1trans = punkttrans(str(z1), 'alk')
            self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", ""))
            z = z1

        z = long('20470000000031000031')
        z1 = long('20470000000001001001')
        ztrans = punkttrans(str(z), 'alk')
        z1trans = punkttrans(str(z1), 'alk')
        self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", "0242"))
        self.col3.appendline(edbs_line(z1trans, ztrans, "", [], "", "0242"))


        z = long('20470000000001000001')
        z1 = long('30570000000001001001')
        ztrans = punkttrans(str(z), 'alk')
        z1trans = punkttrans(str(z1), 'alk')
        self.col3.appendline(edbs_line(ztrans, z1trans, "", [], "", "0242"))
        self.col3.appendline(edbs_line(z1trans, ztrans, "", [], "", "0242"))

        SQL = exportPolygon(self.col3)

        self.assertEqual(SQL, orageom)
        for l in split(SQL,'\n'):
            self.assert_(len(l) < 2500, "Lnge ist %d" % len(l))


class test_exportFeature(unittest.TestCase):
    """ Testet den Export eines Features """

    def setUp(self):

        self.feature = Feature('TEST01')
        # (0 3),(1 2) tempLinie
        line1 = edbs_line((0.0, 3.0),(1.0, 2.0),
                       "",[],"","0242")
        # (1 2),(1 1)
        line2 = edbs_line((1.0, 2.0),(1.0, 1.0),
                       "",[],"","")
        # (1 1), (2 1)
        line3 = edbs_line((1.0, 1.0),(2.0, 1.0) ,
                       "",[],"","")
        # (2 1),(2 2)
        line4 = edbs_line((2.0, 1.0) ,(2.0, 2.0),
                       "",[],"","")
        # (2 2),(1 2)
        line5 = edbs_line((2.0, 2.0),(1.0, 2.0),
                       "",[],"","")
        # (1 2),(0 3) tempLinie
        line6 = edbs_line((1.0, 2.0),(0.0, 3.0),
                       "",[],"","0242")
        # (0 3), (3 3), (3 0), (0 0)
        line7 = edbs_line((0.0, 3.0),(0.0, 0.0),
                       "",[(3.0, 3.0),(3.0, 0.0)],"","")
        # (0 3), (3 3), (0 3)
        line8 = edbs_line((0.0, 3.0),(2.0, 0.0),
                       "",[(3.0, 3.0),(3.0, 0.0)],"","")

        # col1: Linie mit zwei Linienzgen
        self.feature.appendline(line3)
        self.feature.appendline(line1)
        self.feature.appendline(line4)
        self.feature.appendline(line5)
        self.feature.appendline(line6)
        self.feature.appendline(line7)

        self.feature.Objkoor = (1,2)
        self.feature.folie = '001'
        self.feature.art = '0216'
        self.feature.aktualitaet = '01'
        self.feature.typ = 'L'
        self.feature.jahr = '03'
        self.feature.monat = '03'
        self.feature.tag = '09'
        self.feature.datum = '030309'

        self.tableconf = TableConf('land.konf')

    def test_exportFeature(self):
        """Testet den Export des Features durch exportFeature"""


        testSQL = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2006, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1,9,2,1),\nMDSYS.SDO_ORDINATE_ARRAY(1.0,1.0,2.0,1.0,2.0,2.0,1.0,2.0,0.0,3.0,3.0,3.0,3.0,0.0,0.0,0.0));
BEGIN
    INSERT INTO f001_l (objnr, objart, aktualitaet, edatum, geom) VALUES('%s', %s, '%s', to_date('%s', 'RRMMDD'), geom);
    COMMIT;
END;
/
"""\
% (self.feature.Objnr, self.feature.art, self.feature.aktualitaet, self.feature.datum)


        self.assertEqual(\
            exportFeature(self.feature,self.tableconf),
            testSQL)

    def test_exportFeatureBI(self):
        """Testet den Export des Features durch exportFeature"""


        ot = objektTeil()
        ot.setDaten([["13xx  023311bbggggfffzzzzznnn              xx"],[]])

        self.feature.objektteile = [ot,]


        testSQL = """BEGIN
INSERT INTO f001_kennung (objnr, kartentyp, darstellungsart, bundesland, gemarkung, flurnr, zaehler, nenner) VALUES('%s', 'xx', 0233, 'bb', 'gggg', 'fff', 'zzzzz', 'nnn');
COMMIT;
END;
/
DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2006, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1,9,2,1),\nMDSYS.SDO_ORDINATE_ARRAY(1.0,1.0,2.0,1.0,2.0,2.0,1.0,2.0,0.0,3.0,3.0,3.0,3.0,0.0,0.0,0.0));
BEGIN
    INSERT INTO f001_l (objnr, objart, aktualitaet, edatum, fkz, geom) VALUES('%s', %s, '%s', to_date('%s', 'RRMMDD'), 'bbggggfffzzzzznnn', geom);
    COMMIT;
END;
/
"""\
% (self.feature.Objnr, self.feature.Objnr, self.feature.art, self.feature.aktualitaet, self.feature.datum)


        self.assertEqual(\
            exportFeature(self.feature,self.tableconf),
            testSQL)

